module SKEIN_INTERFACE(
   clk, rst_n, 
   load, fetch, 
   idata, odata, 
   busy, start, ack,
   m0, m1, m2, m3, m4, m5, m6, m7,
   hash0, hash1, hash2, hash3,
   hash4, hash5, hash6, hash7,
   Ld_tweak, Ld_posi
);

/************************ input / output *************************/
//{{{
input clk;
input rst_n;

input load;
input fetch;

input [15:0] idata;
output [15:0] odata;

output start;
input busy;

output ack;

output reg [31:0] m0;
output reg [31:0] m1;
output reg [31:0] m2;
output reg [31:0] m3;
output reg [31:0] m4;
output reg [31:0] m5;
output reg [31:0] m6;
output reg [31:0] m7;

output Ld_tweak;
output Ld_posi;

input [31:0] hash0;
input [31:0] hash1;
input [31:0] hash2;
input [31:0] hash3;
input [31:0] hash4;
input [31:0] hash5;
input [31:0] hash6;
input [31:0] hash7;
//}}}

/************************ flipflop wire **************************/
//{{{
reg [5:0] count;
reg [15:0] odata_r;

reg [2:0] state;
reg [2:0] next_state;

reg ack_r;


//}}}

/************************ for output *****************************/
//{{{
assign odata = odata_r;                                     //output
always @(posedge clk or negedge rst_n)begin
   if (~rst_n) odata_r <= 16'h0000;                     //reset
   else begin
      if (state == 3'b011) begin                       //JE^ɑΉďo͎荞ݕω
         if (count == 'h0) odata_r <= hash0[31:16]; 
         else if (count == 'h1) odata_r <= hash0[15:0];
         else if (count == 'h2) odata_r <= hash1[31:16];
         else if (count == 'h3) odata_r <= hash1[15:0];
         else if (count == 'h4) odata_r <= hash2[31:16];
         else if (count == 'h5) odata_r <= hash2[15:0];
         else if (count == 'h6) odata_r <= hash3[31:16];
         else if (count == 'h7) odata_r <= hash3[15:0];
         else if (count == 'h8) odata_r <= hash4[31:16];
         else if (count == 'h9) odata_r <= hash4[15:0];
         else if (count == 'ha) odata_r <= hash5[31:16];
         else if (count == 'hb) odata_r <= hash5[15:0];
         else if (count == 'hc) odata_r <= hash6[31:16];
         else if (count == 'hd) odata_r <= hash6[15:0];
         else if (count == 'he) odata_r <= hash7[31:16];
         else if (count == 'hf) odata_r <= hash7[15:0];
      end
      else odata_r <= odata_r;
   end
end
//}}}

/************************ for input ******************************/
//{{{
always @(posedge clk or negedge rst_n) begin
   if (~rst_n) m0 <= 0;
   else if ((state == 3'b001) && ~count[0]) m0 <= m1;
   else m0 <= m0;
end

always @(posedge clk or negedge rst_n) begin
   if (~rst_n) m1 <= 0;
   else if ((state == 3'b001) && ~count[0]) m1 <= m2;
   else m1 <= m1;
end

always @(posedge clk or negedge rst_n) begin
   if (~rst_n) m2 <= 0;
   else if ((state == 3'b001) && ~count[0]) m2 <= m3;
   else m2 <= m2;
end

always @(posedge clk or negedge rst_n) begin
   if (~rst_n) m3 <= 0;
   else if ((state == 3'b001) && ~count[0]) m3 <= m4;
   else m3 <= m3;
end

always @(posedge clk or negedge rst_n) begin
   if (~rst_n) m4 <= 0;
   else if ((state == 3'b001) && ~count[0]) m4 <= m5;
   else m4 <= m4;
end

always @(posedge clk or negedge rst_n) begin
   if (~rst_n) m5 <= 0;
   else if ((state == 3'b001) && ~count[0]) m5 <= m6;
   else m5 <= m5;
end

always @(posedge clk or negedge rst_n) begin
   if (~rst_n) m6 <= 0;
   else if ((state == 3'b001) && ~count[0]) m6 <= m7;
   else m6 <= m6;
end

always @(posedge clk or negedge rst_n) begin
   if (~rst_n) m7 <= 0;
   else if (state == 3'b001) m7 <= {m7[15:0],idata};
   else m7 <= m7;
end

//}}}

/************************ data count *****************************/
//{{{
always @(posedge clk or negedge rst_n) begin
   if (~rst_n) count <= 0;
   else if ((state == 3'b100) && (count == 6'hf)) count <= 0;
   else if ((state == 3'b001) && (count == 6'h11)) count <= 0;
   else if ((state == 3'b001) || (state == 3'b100)) count <= count + 1;  
   else count <= count;
end
//}}}

/********************* acknowledgement signal ********************/
//{{{
assign ack = ack_r;
always @(posedge clk or negedge rst_n)begin
   if (~rst_n) ack_r <= 0;                           //reset
   else begin
      if (state == 3'b011) ack_r <= 1;               //fetchpack 
      else if (state == 3'b001) ack_r <= 1;          //loadpack //#ǉ
      else ack_r <= 0;
   end
end
//}}}

//{{{ start
assign start = ((state == 3'b010) && ack && (count == 7'h00))? 1:0;
//}}}

/************************ EN signal ******************************/
//{{{
assign Ld_tweak = ((state == 3'b001) && (count == 6'h1))? 1:0;

assign Ld_posi = ((state == 3'b001) && (count == 6'h0))? 1:0;
//}}}

/************************ FSM ************************************/
//{{{
always @(posedge clk or negedge rst_n)begin
	if (~rst_n) state <= 3'b000;
   else state <= next_state;
end

//gH
always @(load or fetch or state or busy) begin    //ꂩ̐MωƂɔ
   case (state)
      3'b000      : begin                         //Xe[g
         if (load) next_state <= 3'b001;                     
         else if (fetch) next_state <= 3'b011;              
         else next_state <= 3'b000;
      end

      3'b001  : next_state <= 3'b010;             //load_ackXe[g ack𐧌

      3'b010  : begin                             //SHA256_coreNXe[g ENHighɂȂ
         if (~busy) next_state <= 3'b000;         //busyHighȂΑҋ@
         else next_state <= 3'b010;
      end
      
      3'b011 : next_state <= 3'b100;              //fetchXe[g odatahash荞 
      
      3'b100  : next_state <= 3'b000;             //cnt <= cnt + 1;

      default: next_state <= 3'b000;
   endcase
end
//}}}


endmodule 
// vim:fdm=marker
